extends GutTest

# _drag_node spawns in the top-left corner with the size 128x128
var _drag_node: Control
var _sender

func before_all():
	_drag_node = preload("res://DragTest.tscn").instantiate()
	add_child(_drag_node)
	_sender = InputSender.new(Input)
	print(_drag_node)
func after_each():
	_sender.clear()
func after_all():
	_drag_node.queue_free()

# This test tries to use exclusively InputSender to press, drag,
# and release the control.
func test_drag_using_input_sender():
	await (_sender.mouse_set_position(Vector2(64,64)) # Set position to middle of control
		.mouse_left_button_down().wait_secs(1) # Begin click
		.mouse_relative_motion(Vector2(128,0)) # Move to the right to begin drag
	).wait_secs(1).idle
	
	assert_not_null(get_viewport().gui_get_drag_data(), "Control is being dragged")
	
	await _sender.mouse_left_button_up().wait_frames(1).idle # Release the mouse
	
	assert_null(get_viewport().gui_get_drag_data(), "Control is no longer being dragged")

# This test uses InputSender to press and release the mouse button,
# but generates a drag event with InputFactory
func test_drag_using_input_factory():
	await _sender.mouse_left_button_down(Vector2(64,64)).wait_secs(1) # Mouse down on control
	
	var event = InputFactory.mouse_motion(Vector2(196,64)) # Create a mouse motion event at the destination
	event.relative = Vector2i(128,0) # Set the relative motion
	event.button_mask = MOUSE_BUTTON_MASK_LEFT # Indicate we are holding left click
	
	await _sender.send_event(event).wait_secs(1).idle # Drag event
	
	assert_not_null(get_viewport().gui_get_drag_data(), "Control is being dragged")
	
	await _sender.mouse_left_button_up(Vector2(196,64)).wait_frames(1).idle # Release the mouse
	
	assert_null(get_viewport().gui_get_drag_data(), "Control is no longer being dragged")
